@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2016 - 2022 -- Lars Heuer
+# Copyright (c) 2016 - 2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -16,10 +16,10 @@ from itertools import chain
 import shutil
 import nox
 
-_PY_VERSIONS = ('2.7', '3.7', '3.8', '3.9', '3.10', 'pypy', 'pypy3')
-_PY_DEFAULT_VERSION = '3.10'
+_PY_VERSIONS = ('3.5', '3.6', '3.7', '3.8', '3.9', '3.10', '3.11', '3.12', 'pypy3')
+_PY_DEFAULT_VERSION = sys.version[:4]
 
-nox.options.sessions = ['test-2.7', 'test-{}'.format(_PY_DEFAULT_VERSION), 'test-pypy', 'test-pypy3']
+nox.options.sessions = chain(['test-{}'.format(version) for version in _PY_VERSIONS], ['coverage', 'lint'])
 
 
 @nox.session(python=_PY_VERSIONS)
@@ -27,15 +27,7 @@ def test(session):
     """\
     Run test suite.
     """
-    if session.python == 'pypy':
-        # See <https://github.com/heuer/segno/issues/80>
-        pip_ver = session.run('pip', '--version', silent=True)
-        m = re.search(r'pip ([0-9.]{4})', pip_ver)
-        assert m
-        if m.group(1) in ('20.2', '20.3'):
-            session.run('pip', 'uninstall', '-y', 'pip')
-            session.run('easy_install', 'pip==20.1.1')
-    session.install('-Ur', 'requirements-testing.txt')
+    session.install('-Ur', 'tests/requirements.txt')
     session.install('.')
     session.run('py.test')
 
@@ -45,7 +37,7 @@ def docs(session):
     """\
     Build the documentation.
     """
-    session.install('-Ur', 'requirements-rtd.txt')
+    session.install('-Ur', 'docs/requirements.txt')
     output_dir = os.path.abspath(os.path.join(session.create_tmp(), 'output'))
     shutil.rmtree(output_dir, ignore_errors=True)
     doctrees, html, man = map(partial(os.path.join, output_dir), ['doctrees', 'html', 'man'])
@@ -67,14 +59,10 @@ def coverage(session):
     """\
     Run coverage.
     """
-    session.install('coverage', '-Ur', 'requirements-testing.txt')
+    session.install('-Ur', 'tests/requirements.txt')
     session.install('.')
-    output_dir = os.path.abspath(os.path.join(session.create_tmp(), 'html'))
-    cover = partial(session.run, 'coverage')
-    cover('erase')
-    cover('run', './tests/alltests.py')
-    cover('report')
-    cover('html', '-d', output_dir)
+    html_output_dir = os.path.abspath(os.path.join(session.create_tmp(), 'html'))
+    session.run('pytest', '--cov=segno', '--cov-report=term', '--cov-report=html:%s' % html_output_dir)
 
 
 @nox.session(python=_PY_DEFAULT_VERSION)
@@ -85,7 +73,7 @@ def lint(session):
     session.install('flake8', 'mypy')
     session.run('mypy', 'segno')
     session.run('flake8', 'segno')
-    session.run('flake8', 'tests/')
+    session.run('flake8', 'tests')
 
 
 @nox.session(python=_PY_DEFAULT_VERSION)
@@ -93,7 +81,8 @@ def benchmarks(session):
     """\
     Run the benchmarks and create the charts.
     """
-    session.install('qrcode', 'Pillow', 'pyqrcode', 'pypng', 'qrcodegen', 'pygal')
+    session.install('qrcode', 'Pillow', 'qrcodegen', 'pygal')
+    session.install('setuptools')  # Required by pygal since it uses pkg_resources
     session.install('.')
     sandbox_dir = os.path.join(os.path.dirname(__file__), 'sandbox')
     session.run('python', os.path.join(sandbox_dir, 'benchmarks.py'))
