@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2016 - 2022 -- Lars Heuer
+# Copyright (c) 2016 - 2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -10,27 +10,18 @@ QR Code and Micro QR Code implementation.
 
 "QR Code" and "Micro QR Code" are registered trademarks of DENSO WAVE INCORPORATED.
 """
-from __future__ import absolute_import, unicode_literals
 import sys
 import io
 from . import encoder
 from .encoder import DataOverflowError
 from . import writers, utils
-try:  # pragma: no cover
-    str_type = basestring  # noqa: F821
-except NameError:  # pragma: no cover
-    str_type = str
 
-__version__ = '1.5.2'
+__version__ = '1.6.0'
 
 __all__ = ('make', 'make_qr', 'make_micro', 'make_sequence', 'QRCode',
            'QRCodeSequence', 'DataOverflowError')
 
 
-# <https://wiki.python.org/moin/PortingToPy3k/BilingualQuickRef#New_Style_Classes>
-__metaclass__ = type
-
-
 def make(content, error=None, version=None, mode=None, mask=None, encoding=None,
          eci=False, micro=None, boost_error=True):
     """\
@@ -223,7 +214,7 @@ class QRCode:
     """\
     Represents a (Micro) QR Code.
     """
-    __slots__ = ('matrix', 'mask', '_version', '_error', '_mode')
+    __slots__ = ('matrix', 'mask', '_version', '_error', '_mode', '_matrix_size')
 
     def __init__(self, code):
         """\
@@ -232,7 +223,8 @@ class QRCode:
         :param code: An object with a ``matrix``, ``version``, ``error``,
             ``mask`` and ``segments`` attribute.
         """
-        self.matrix = code.matrix
+        matrix = code.matrix
+        self.matrix = matrix
         """Returns the matrix.
 
         :rtype: tuple of :py:class:`bytearray` instances.
@@ -242,6 +234,7 @@ class QRCode:
 
         :rtype: int
         """
+        self._matrix_size = len(matrix[0]), len(matrix)
         self._version = code.version
         self._error = code.error
         self._mode = code.segments[0].mode if len(code.segments) == 1 else None
@@ -302,7 +295,7 @@ class QRCode:
 
         :rtype: int
         """
-        return utils.get_default_border_size(self._version)
+        return utils.get_default_border_size(self._matrix_size)
 
     @property
     def is_micro(self):
@@ -333,7 +326,7 @@ class QRCode:
                 default quiet zone (4 for QR Codes, 2 for Micro QR Codes).
         :rtype: tuple (width, height)
         """
-        return utils.get_symbol_size(self._version, scale=scale, border=border)
+        return utils.get_symbol_size(self._matrix_size, scale=scale, border=border)
 
     def matrix_iter(self, scale=1, border=None, verbose=False):
         """\
@@ -383,7 +376,7 @@ class QRCode:
                 invalid (i.e. negative).
         """
         iterfn = utils.matrix_iter_verbose if verbose else utils.matrix_iter
-        return iterfn(self.matrix, self._version, scale, border)
+        return iterfn(self.matrix, self._matrix_size, scale, border)
 
     def show(self, delete_after=20, scale=10, border=None, dark='#000',
              light='#fff'):  # pragma: no cover
@@ -418,12 +411,8 @@ class QRCode:
         import tempfile
         import webbrowser
         import threading
-        try:  # Python 3
-            from urllib.parse import urljoin
-            from urllib.request import pathname2url
-        except ImportError:  # Python 2
-            from urlparse import urljoin  # noqa
-            from urllib import pathname2url  # noqa
+        from urllib.parse import urljoin
+        from urllib.request import pathname2url
 
         def delete_file(name):
             time.sleep(delete_after)
@@ -477,7 +466,7 @@ class QRCode:
                         (default: ``False``)
         :rtype: str
         """
-        return writers.as_svg_data_uri(self.matrix, self._version,
+        return writers.as_svg_data_uri(self.matrix, self._matrix_size,
                                        xmldecl=xmldecl, nl=nl,
                                        encode_minimal=encode_minimal,
                                        omit_charset=omit_charset, **kw)
@@ -520,7 +509,7 @@ class QRCode:
 
         :rtype: str
         """
-        return writers.as_png_data_uri(self.matrix, self._version, **kw)
+        return writers.as_png_data_uri(self.matrix, self._matrix_size, **kw)
 
     def terminal(self, out=None, border=None, compact=False):
         """\
@@ -540,18 +529,18 @@ class QRCode:
                 (default: ``False``).
         """
         if compact:
-            writers.write_terminal_compact(self.matrix, self._version, out or sys.stdout, border)
+            writers.write_terminal_compact(self.matrix, self._matrix_size, out or sys.stdout, border)
         elif out is None and sys.platform == 'win32':  # pragma: no cover
             # Windows < 10 does not support ANSI escape sequences, try to
             # call the a Windows specific terminal output which uses the
             # Windows API.
             try:
-                writers.write_terminal_win(self.matrix, self._version, border)
+                writers.write_terminal_win(self.matrix, self._matrix_size, border)
             except OSError:
                 # Use the standard output even if it may print garbage
-                writers.write_terminal(self.matrix, self._version, sys.stdout, border)
+                writers.write_terminal(self.matrix, self._matrix_size, sys.stdout, border)
         else:
-            writers.write_terminal(self.matrix, self._version, out or sys.stdout, border)
+            writers.write_terminal(self.matrix, self._matrix_size, out or sys.stdout, border)
 
     def save(self, out, kind=None, **kw):
         """\
@@ -949,7 +938,7 @@ class QRCode:
                 insensitive.
         :param kw: Any of the supported keywords by the specific serializer.
         """
-        writers.save(self.matrix, self._version, out, kind, **kw)
+        writers.save(self.matrix, self._matrix_size, out, kind, **kw)
 
     def __getattr__(self, name):
         """\
@@ -961,10 +950,15 @@ class QRCode:
         of invoking the function depends on the plugin.
         """
         if name.startswith('to_'):
-            from pkg_resources import iter_entry_points
+            try:
+                # Try to use the 3rd party lib first. This is required for
+                # Python versions < 3.10
+                import importlib_metadata as metadata
+            except ImportError:
+                from importlib import metadata
             from functools import partial
-            for ep in iter_entry_points(group='segno.plugin.converter',
-                                        name=name[3:]):
+            for ep in metadata.entry_points(group='segno.plugin.converter',
+                                            name=name[3:]):
                 plugin = ep.load()
                 return partial(plugin, self)
         raise AttributeError('{0} object has no attribute {1}'
@@ -1008,7 +1002,7 @@ class QRCodeSequence(tuple):
         """
         filename = lambda o, n: o  # noqa: E731
         m = len(self)
-        if m > 1 and isinstance(out, str_type):
+        if m > 1 and isinstance(out, str):
             dot_idx = out.rfind('.')
             if dot_idx > -1:
                 out = out[:dot_idx] + '-{0:02d}-{1:02d}' + out[dot_idx:]
