@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2016 - 2021 -- Lars Heuer
+# Copyright (c) 2016 - 2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -12,7 +12,6 @@ DOES NOT belong to the public API.
 
 "QR Code" and "Micro QR Code" are registered trademarks of DENSO WAVE INCORPORATED.
 """
-from __future__ import absolute_import, division
 from operator import itemgetter, gt, lt, xor
 from functools import partial, reduce
 from itertools import islice, chain, product
@@ -21,28 +20,13 @@ import math
 import codecs
 from collections import namedtuple
 from . import consts
-_PY2 = False
-try:  # pragma: no cover
-    from itertools import zip_longest
-    str_type = str  # type: ignore
-    numeric = int
-except ImportError:  # pragma: no cover
-    _PY2 = True
-    from itertools import izip_longest as zip_longest, imap as map  # type: ignore
-    str_type = basestring  # type: ignore  # noqa: F821
-    from numbers import Number
-    numeric = Number  # type: ignore
-    str = unicode  # type: ignore  # noqa: F821
-    range = xrange  # type: ignore  # noqa: F821
+from itertools import zip_longest
 import sys  # noqa: E402
 _MAX_PENALTY_SCORE = sys.maxsize
 del sys
 
 __all__ = ('encode', 'encode_sequence', 'DataOverflowError')
 
-# <https://wiki.python.org/moin/PortingToPy3k/BilingualQuickRef#New_Style_Classes>
-__metaclass__ = type
-
 
 class DataOverflowError(ValueError):
     """\
@@ -255,16 +239,18 @@ def _encode(segments, error, version, mask, eci, boost_error, sa_info=None):
     # ISO/IEC 18004:2015(E) -- 7.6 Constructing the final message codeword sequence (page 45)
     buff = make_final_message(version, error, buff)
     # Matrix with timing pattern and reserved format / version regions
-    matrix = make_matrix(version)
+    width = calc_matrix_size(version)
+    height = width
+    matrix = make_matrix(width, height)
     # ISO/IEC 18004:2015 -- 6.3.3 Finder pattern (page 16)
-    add_finder_patterns(matrix, is_micro)
+    add_finder_patterns(matrix, width, height)
     # ISO/IEC 18004:2015 -- 6.3.6 Alignment patterns (page 17)
-    add_alignment_patterns(matrix, version)
+    add_alignment_patterns(matrix, width, height)
     # ISO/IEC 18004:2015 -- 7.7 Codeword placement in matrix (page 46)
     add_codewords(matrix, buff, version)
     # ISO/IEC 18004:2015(E) -- 7.8.2 Data mask patterns (page 50)
     # ISO/IEC 18004:2015(E) -- 7.8.3 Evaluation of data masking results (page 53)
-    mask, matrix = find_and_apply_best_mask(matrix, version, is_micro, mask)
+    mask, matrix = find_and_apply_best_mask(matrix, width, height, mask)
     # ISO/IEC 18004:2015(E) -- 7.9 Format information (page 55)
     add_format_info(matrix, version, error, mask)
     # ISO/IEC 18004:2015(E) -- 7.10 Version information (page 58)
@@ -392,7 +378,19 @@ def write_pad_codewords(buff, version, capacity, length):
             write(pad_codewords[i % 2])
 
 
-def add_finder_patterns(matrix, is_micro):
+# Finder pattern (includes separator around each side!)
+_FINDER_PATTERN = ((0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0),
+                   (0x0, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x0),
+                   (0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0),
+                   (0x0, 0x1, 0x0, 0x1, 0x1, 0x1, 0x0, 0x1, 0x0),
+                   (0x0, 0x1, 0x0, 0x1, 0x1, 0x1, 0x0, 0x1, 0x0),
+                   (0x0, 0x1, 0x0, 0x1, 0x1, 0x1, 0x0, 0x1, 0x0),
+                   (0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0),
+                   (0x0, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x0),
+                   (0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0))
+
+
+def add_finder_patterns(matrix, width, height):
     """\
     Adds the finder pattern(s) with the separators to the matrix.
 
@@ -403,27 +401,18 @@ def add_finder_patterns(matrix, is_micro):
     ISO/IEC 18004:2015(E) -- 6.3.4 Separator (page 17)
 
     :param matrix: The matrix.
-    :param bool is_micro: Indicates if the matrix represents a Micro QR Code.
+    :param tuple(int, int) matrix_size: Tuple of width and height of the matrix.
     """
-    # Finder pattern (includes separator around each side!)
-    pattern = ((0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0),
-               (0x0, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x0),
-               (0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0),
-               (0x0, 0x1, 0x0, 0x1, 0x1, 0x1, 0x0, 0x1, 0x0),
-               (0x0, 0x1, 0x0, 0x1, 0x1, 0x1, 0x0, 0x1, 0x0),
-               (0x0, 0x1, 0x0, 0x1, 0x1, 0x1, 0x0, 0x1, 0x0),
-               (0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0),
-               (0x0, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x0),
-               (0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0))
+    is_square = width == height
     corners = ((0, 0), (0, len(matrix) - 8), (-8, 0))  # Upper left, upper right, bottom left
-    if is_micro:
+    if is_square and width < 21:
         corners = ((0, 0),)
     finder_range = range(8)
     for i, j in corners:
         offset = 1 if i == 0 else 0
         sepoffset = 0 if j != 0 else 1
         for r in finder_range:
-            matrix[i + r][j:j + 8] = pattern[offset + r][sepoffset:sepoffset + 8]
+            matrix[i + r][j:j + 8] = _FINDER_PATTERN[offset + r][sepoffset:sepoffset + 8]
 
 
 def add_timing_pattern(matrix, is_micro):
@@ -445,15 +434,20 @@ def add_timing_pattern(matrix, is_micro):
         bit ^= 0x1
 
 
-def add_alignment_patterns(matrix, version):
+def add_alignment_patterns(matrix, width, height):
     """\
     Adds the adjustment patterns to the matrix. For versions < 2 this is a
     no-op.
 
     ISO/IEC 18004:2015(E) -- 6.3.6 Alignment patterns (page 17)
     ISO/IEC 18004:2015(E) -- Annex E Position of alignment patterns (page 83)
+
+    :param matrix: An iterable of bytearrays.
+    :param tuple(int, int) matrix_size: Tuple of width and height of the matrix.
     """
-    if version < 2:
+    is_square = width == height
+    version = (width - 17) // 4  # QR Codes: version * 4 +  17 == width / height of the matrix w/o border
+    if is_square and version < 2:  # QR Codes version < 2 don't have alignment patterns
         return
     pattern = (0x1, 0x1, 0x1, 0x1, 0x1,
                0x1, 0x0, 0x0, 0x0, 0x1,
@@ -598,7 +592,7 @@ def make_blocks(ec_infos, buff):
     return data_blocks, error_blocks
 
 
-def find_and_apply_best_mask(matrix, version, is_micro, proposed_mask=None):
+def find_and_apply_best_mask(matrix, width, height, proposed_mask=None):
     """\
     Applies all mask patterns against the provided QR Code matrix and returns
     the best matrix and best pattern.
@@ -609,19 +603,18 @@ def find_and_apply_best_mask(matrix, version, is_micro, proposed_mask=None):
     ISO/IEC 18004:2015(E) -- 7.8.3.2 Evaluation of Micro QR Code symbols (page 54/55)
 
     :param matrix: A matrix.
-    :param int version: A version (Micro) QR Code version constant.
-    :param bool is_micro: Indicates if the matrix represents a Micro QR Code
+    :param tuple(int, int) matrix_size: Tuple of width and height of the matrix.
     :param proposed_mask: Optional int to indicate the preferred mask.
     :rtype: tuple
     :return: A tuple of the best matrix and best data mask pattern index.
     """
-    matrix_size = len(matrix)
     # ISO/IEC 18004:2015 -- 7.8.3.1 Evaluation of QR Code symbols (page 53/54)
     # The data mask pattern which results in the lowest penalty score shall
     # be selected for the symbol.
     is_better = lt
     best_score = _MAX_PENALTY_SCORE
     eval_mask = evaluate_mask
+    is_micro = width == height and width < 21
     if is_micro:
         # ISO/IEC 18004:2015(E) - 7.8.3.2 Evaluation of Micro QR Code symbols (page 54/55)
         # The data mask pattern which results in the highest score shall be
@@ -631,9 +624,9 @@ def find_and_apply_best_mask(matrix, version, is_micro, proposed_mask=None):
         eval_mask = evaluate_micro_mask
     # Matrix to check if a module belongs to the encoding region
     # or to the function patterns
-    function_matrix = make_matrix(version)
-    add_finder_patterns(function_matrix, is_micro)
-    add_alignment_patterns(function_matrix, version)
+    function_matrix = make_matrix(width, height)
+    add_finder_patterns(function_matrix, width, height)
+    add_alignment_patterns(function_matrix, width, height)
     if not is_micro:
         function_matrix[-8][8] = 0x1
 
@@ -643,17 +636,17 @@ def find_and_apply_best_mask(matrix, version, is_micro, proposed_mask=None):
     mask_patterns = get_data_mask_functions(is_micro)
     # If the user supplied a mask pattern, the evaluation step is skipped
     if proposed_mask is not None:
-        apply_mask(matrix, mask_patterns[proposed_mask], matrix_size,
+        apply_mask(matrix, mask_patterns[proposed_mask], width, height,
                    is_encoding_region)
         return proposed_mask, matrix
 
     best_matrix = None
     for mask_number, mask_pattern in enumerate(mask_patterns):
         m = [ba[:] for ba in matrix]
-        apply_mask(m, mask_pattern, matrix_size, is_encoding_region)
+        apply_mask(m, mask_pattern, width, height, is_encoding_region)
         # NOTE: DO NOT add format / version info in advance of evaluation
         # See ISO/IEC 18004:2015(E) -- 7.8. Data masking (page 50)
-        score = eval_mask(m, matrix_size)
+        score = eval_mask(m, width, height)
         if is_better(score, best_score):
             best_score = score
             best_pattern = mask_number
@@ -661,7 +654,7 @@ def find_and_apply_best_mask(matrix, version, is_micro, proposed_mask=None):
     return best_pattern, best_matrix
 
 
-def apply_mask(matrix, mask_pattern, matrix_size, is_encoding_region):
+def apply_mask(matrix, mask_pattern, width, height, is_encoding_region):
     """\
     Applies the provided mask pattern on the `matrix`.
 
@@ -673,15 +666,15 @@ def apply_mask(matrix, mask_pattern, matrix_size, is_encoding_region):
     :param is_encoding_region: A function which returns ``True`` iff the
             row index / col index belongs to the data region.
     """
-    module_range = range(matrix_size)
-    for i in module_range:
+    width_range = range(width)
+    for i in range(height):
         row = matrix[i]
-        for j in module_range:
+        for j in width_range:
             if is_encoding_region(i, j):
                 row[j] ^= mask_pattern(i, j)
 
 
-def evaluate_mask(matrix, matrix_size):
+def evaluate_mask(matrix, width, height):
     """\
     Evaluates the provided `matrix` of a QR code.
 
@@ -691,10 +684,10 @@ def evaluate_mask(matrix, matrix_size):
     :param matrix_size: The width (or height) of the matrix.
     :return int: The penalty score of the matrix.
     """
-    return sum(mask_scores(matrix, matrix_size))
+    return sum(mask_scores(matrix, width, height))
 
 
-def mask_scores(matrix, matrix_size):
+def mask_scores(matrix, width, height):
     """\
     Returns the penalty score features of the matrix.
 
@@ -732,9 +725,9 @@ def mask_scores(matrix, matrix_size):
         idx = seq.find(n3_pattern)
         while idx != -1:
             offset = idx + 7
-            if idx in (0, matrix_size - 7) \
-                    or not any(seq[max(idx - 4, 0):min(idx, matrix_size)]) \
-                    or not any(seq[max(offset, 0):min(offset + 4, matrix_size)]):
+            if idx in (0, qr_size - 7) \
+                    or not any(seq[max(idx - 4, 0):min(idx, qr_size)]) \
+                    or not any(seq[max(offset, 0):min(offset + 4, qr_size)]):
                 count += 40  # N3 = 40
             else:
                 # Found no / not enough light modules, start at next possible
@@ -749,19 +742,21 @@ def mask_scores(matrix, matrix_size):
     score_n1 = 0
     score_n2 = 0
     score_n3 = 0
-    module_range = range(matrix_size)
+    assert width == height
+    qr_size = width
+    qr_module_range = range(qr_size)
     dark_module_counter = 0
     last_row = None
     # Collects the bytes column-wise (required to calculate score N3)
-    n3_column = bytearray(matrix_size)
-    for i in module_range:
+    n3_column = bytearray(qr_size)
+    for i in qr_module_range:
         row = matrix[i]
         row_prev_bit = -1
         col_prev_bit = -1
         # N1
         n1_row_counter = 0
         n1_col_counter = 0
-        for j in module_range:
+        for j in qr_module_range:
             row_current_bit = row[j]
             col_current_bit = matrix[j][i]
             n3_column[j] = col_current_bit
@@ -795,12 +790,12 @@ def mask_scores(matrix, matrix_size):
         if n1_col_counter >= 5:
             score_n1 += n1_col_counter - 2
     # N4
-    percent = float(dark_module_counter) / (matrix_size ** 2)
+    percent = float(dark_module_counter) / (qr_size ** 2)
     score_n4 = 10 * int(abs(percent * 100 - 50) / 5)  # N4 = 10
     return score_n1, score_n2, score_n3, score_n4
 
 
-def evaluate_micro_mask(matrix, matrix_size):
+def evaluate_micro_mask(matrix, width, height):
     """\
     Evaluates the provided `matrix` of a Micro QR code.
 
@@ -810,7 +805,7 @@ def evaluate_micro_mask(matrix, matrix_size):
     :param matrix_size: The width (or height) of the matrix.
     :return int: The penalty score of the matrix.
     """
-    module_range = range(1, matrix_size)
+    module_range = range(1, width)
     last_row = matrix[-1]
     sum1 = sum(matrix[i][-1] for i in module_range)
     sum2 = sum(last_row[i] for i in module_range)
@@ -962,7 +957,7 @@ def prepare_data(content, mode, encoding):
     """
     segments = Segments()
     add_segment = segments.add_segment
-    if isinstance(content, (str_type, bytes, numeric)):
+    if isinstance(content, (str, bytes, int)):
         add_segment(make_segment(content, mode, encoding))
         return segments
     for item in content:
@@ -1075,14 +1070,10 @@ def make_segment(data, mode, encoding=None):
                 append_bits(to_byte(chunk), 6)
     elif segment_mode == consts.MODE_BYTE:
         # ISO/IEC 18004:2015(E) -- 7.4.5 Byte mode (page 27)
-        if _PY2:  # pragma: no cover
-            segment_data = (ord(b) for b in segment_data)
         for b in segment_data:
             append_bits(b, 8)
     elif segment_mode == consts.MODE_HANZI:
         # GBT 18284-2000 -- 6.4.5 Hanzi mode (page 18)
-        if _PY2:  # pragma: no cover
-            segment_data = [ord(b) for b in segment_data]
         # Note: len(segment.data)! segment.data_length = len(segment.data) / 2!!
         for i in range(0, segment_length, 2):
             code = (segment_data[i] << 8) | segment_data[i + 1]
@@ -1102,8 +1093,6 @@ def make_segment(data, mode, encoding=None):
             append_bits(((diff >> 8) * 0x60) + (diff & 0xff), 13)
     else:
         # ISO/IEC 18004:2015(E) -- 7.4.6 Kanji mode (page 29)
-        if _PY2:  # pragma: no cover
-            segment_data = [ord(b) for b in segment_data]
         for i in range(0, segment_length, 2):
             code = (segment_data[i] << 8) | segment_data[i + 1]
             if 0x8140 <= code <= 0x9ffc:
@@ -1123,7 +1112,7 @@ def make_segment(data, mode, encoding=None):
     return _Segment(buff.getbits(), char_count, segment_mode, segment_encoding)
 
 
-def make_matrix(version, reserve_regions=True, add_timing=True):
+def make_matrix(width, height, reserve_regions=True, add_timing=True):
     """\
     Creates a matrix of the provided `size` (w x h) initialized with the
     (illegal) value 0x2.
@@ -1131,14 +1120,16 @@ def make_matrix(version, reserve_regions=True, add_timing=True):
     The "timing pattern" is already added to the matrix and the version
     and format areas are initialized with 0x0.
 
-    :param int version: The (Micro) QR Code version
+    :param int width: Matrix width
+    :param int height: Matrix height.
     :rtype: tuple of bytearrays
     """
-    size = calc_matrix_size(version)
-    row = [0x2] * size
-    matrix = tuple([bytearray(row) for i in range(size)])
+    is_square = width == height
+    is_micro = is_square and width < 21
+    row = [0x2] * width
+    matrix = tuple(bytearray(row) for i in range(height))
     if reserve_regions:
-        if version > 6:
+        if is_square and width > 41:  # QR Codes < version 7 don't have a version pattern
             # Reserve version pattern areas
             for i in range(6):
                 row = matrix[i]
@@ -1155,12 +1146,12 @@ def make_matrix(version, reserve_regions=True, add_timing=True):
         for i in range(9):
             matrix[i][8] = 0x0  # Upper left
             row_eight[i] = 0x0  # Upper bottom
-            if version > 0:
+            if not is_micro:
                 matrix[-i][8] = 0x0  # Bottom left
                 row_eight[-i] = 0x0  # Upper right
     if add_timing:
         # ISO/IEC 18004:2015 -- 6.3.5 Timing pattern (page 17)
-        add_timing_pattern(matrix, version < 1)
+        add_timing_pattern(matrix, is_micro)
     return matrix
 
 
@@ -1345,8 +1336,6 @@ def is_kanji(data):
     data_len = len(data)
     if not data_len or data_len % 2:
         return False
-    if _PY2:  # pragma: no cover
-        data = (ord(c) for c in data)
     data_iter = iter(data)
     for i in range(0, data_len, 2):
         code = (next(data_iter) << 8) | next(data_iter)
@@ -1431,7 +1420,7 @@ def calc_structured_append_parity(content):
     :param str content: The content.
     :rtype: int
     """
-    if not isinstance(content, str_type):
+    if not isinstance(content, str):
         content = str(content)
     try:
         data = content.encode('iso-8859-1')
@@ -1440,8 +1429,6 @@ def calc_structured_append_parity(content):
             data = content.encode('shift-jis')
         except (LookupError, UnicodeError):
             data = content.encode('utf-8')
-    if _PY2:  # pragma: no cover
-        data = (ord(c) for c in data)
     return reduce(xor, data)
 
 
