@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2016 - 2022 -- Lars Heuer
+# Copyright (c) 2016 - 2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -566,143 +566,139 @@ def test_find_version(data, error, micro, expected_version):
 def test_thonky_add_format_info():
     # <http://www.thonky.com/qr-code-tutorial/format-version-information#put-the-format-string-into-the-qr-code>
     version = 1
-    matrix = encoder.make_matrix(version, reserve_regions=False)
-    encoder.add_finder_patterns(matrix, is_micro=False)
+    width, height = 21, 21  # Version 1
+    matrix = encoder.make_matrix(width, height, reserve_regions=False)
+    encoder.add_finder_patterns(matrix, width, height)
     encoder.add_format_info(matrix, version, consts.ERROR_LEVEL_L, 4)
-    ref_matrix = read_matrix('thonky_format')
+    ref_matrix, ref_matrix_width, ref_matrix_height = read_matrix('thonky_format')
+    assert ref_matrix_width, ref_matrix_height == (width, height)
     assert ref_matrix == matrix
 
 
 def test_thonky_add_version_info():
     # <http://www.thonky.com/qr-code-tutorial/format-version-information>
     version = 7
-    matrix = encoder.make_matrix(version, reserve_regions=False)
-    encoder.add_finder_patterns(matrix, is_micro=False)
-    encoder.add_alignment_patterns(matrix, version)
+    width, height = 45, 45  # Version 7
+    matrix = encoder.make_matrix(width, height, reserve_regions=False)
+    encoder.add_finder_patterns(matrix, width, height)
+    encoder.add_alignment_patterns(matrix, width, height)
     encoder.add_version_info(matrix, version)
     matrix[-8][8] = 0x1  # dark module
-    ref_matrix = read_matrix('thonky_version')
+    ref_matrix, ref_matrix_width, ref_matrix_height = read_matrix('thonky_version')
+    assert ref_matrix_width, ref_matrix_height == (width, height)
     assert ref_matrix == matrix
 
 
 def test_eval_micro():
     # ISO/IEC 18004:2006(E) page 54
     # 6.8.2.2 Evaluation of Micro QR Code symbols
-    matrix = read_matrix('iso_6.8.2.2')
-    res = encoder.evaluate_micro_mask(matrix, len(matrix))
+    matrix, width, height = read_matrix('iso_6.8.2.2')
+    res = encoder.evaluate_micro_mask(matrix, width, height)
     assert 104 == res
 
 
 def make_thonky_score_matrix():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking-1')
-    return matrix, len(matrix)
+    return read_matrix('thonky_datamasking-1')
 
 
 def test_score_n1():
-    matrix, matrix_size = make_thonky_score_matrix()
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = make_thonky_score_matrix()
+    scores = encoder.mask_scores(matrix, width, height)
     score = scores[0]
     assert 180 == score
 
 
 def test_score_n2():
-    matrix, matrix_size = make_thonky_score_matrix()
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = make_thonky_score_matrix()
+    scores = encoder.mask_scores(matrix, width, height)
     score = scores[1]
     assert 90 == score
 
 
 def test_score_n3():
-    matrix, matrix_size = make_thonky_score_matrix()
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = make_thonky_score_matrix()
+    scores = encoder.mask_scores(matrix, width, height)
     score = scores[2]
     # Thonky: 80
     assert 760 == score
 
 
 def test_score_n4():
-    matrix = read_matrix('thonky_datamasking-2')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking-2')
+    scores = encoder.mask_scores(matrix, width, height)
     score = scores[3]
     assert 0 == score
 
 
 def test_thonky_pattern_0():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking_mask-0')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking_mask-0')
+    scores = encoder.mask_scores(matrix, width, height)
     assert 180 == scores[0]
     assert 90 == scores[1]
     # Thonky: 80
     assert 760 == scores[2]
     assert 0 == scores[3]
     # See score 3
-    assert 350 - 80 + 760 == encoder.evaluate_mask(matrix, matrix_size)
+    assert 350 - 80 + 760 == encoder.evaluate_mask(matrix, width, height)
 
 
 def test_thonky_pattern_1():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking_mask-1')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking_mask-1')
+    scores = encoder.mask_scores(matrix, width, height)
     assert 172 == scores[0]
     assert 129 == scores[1]
     # Thonky: 120
     assert 760 == scores[2]
     assert 0 == scores[3]
     # See score 3
-    assert 421 - 120 + 760 == encoder.evaluate_mask(matrix, matrix_size)
+    assert 421 - 120 + 760 == encoder.evaluate_mask(matrix, width, height)
 
 
 def test_thonky_pattern_2():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking_mask-2')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking_mask-2')
+    scores = encoder.mask_scores(matrix, width, height)
     assert 206 == scores[0]
     assert 141 == scores[1]
     # Thonky: 160
     assert 800 == scores[2]
     assert 0 == scores[3]
-    assert 507 - 160 + 800 == encoder.evaluate_mask(matrix, matrix_size)
+    assert 507 - 160 + 800 == encoder.evaluate_mask(matrix, width, height)
 
 
 def test_thonky_pattern_3():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking_mask-3')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking_mask-3')
+    scores = encoder.mask_scores(matrix, width, height)
     assert 180 == scores[0]
     assert 141 == scores[1]
     # Thonky: 120
     assert 760 == scores[2]
     # Thonky: 2, but that's impossible: Either 0 or a multiple of 10 (N4 = 10)
     assert 0 == scores[3]
-    assert 443 - 2 - 120 + 760 == encoder.evaluate_mask(matrix, matrix_size)
+    assert 443 - 2 - 120 + 760 == encoder.evaluate_mask(matrix, width, height)
 
 
 def test_thonky_pattern_4():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking_mask-4')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking_mask-4')
+    scores = encoder.mask_scores(matrix, width, height)
     assert 195 == scores[0]
     assert 138 == scores[1]
     # Thonky: 200
     assert 800 == scores[2]
     assert 0 == scores[3]
     # See score 3
-    assert 533 - 200 + 800 == encoder.evaluate_mask(matrix, matrix_size)
+    assert 533 - 200 + 800 == encoder.evaluate_mask(matrix, width, height)
 
 
 def test_thonky_pattern_5():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking_mask-5')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking_mask-5')
+    scores = encoder.mask_scores(matrix, width, height)
     assert 189 == scores[0]
     assert 156 == scores[1]
     # Thonky: 200
@@ -710,14 +706,13 @@ def test_thonky_pattern_5():
     # Thonky: 2, but that's impossible: Either 0 or a multiple of 10 (N4 = 10)
     assert 0 == scores[3]
     # See score 3 and 4
-    assert 547 - 2 - 200 + 800 == encoder.evaluate_mask(matrix, matrix_size)
+    assert 547 - 2 - 200 + 800 == encoder.evaluate_mask(matrix, width, height)
 
 
 def test_thonky_pattern_6():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking_mask-6')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking_mask-6')
+    scores = encoder.mask_scores(matrix, width, height)
     assert 171 == scores[0]
     assert 102 == scores[1]
     # Thonky: 80
@@ -725,29 +720,28 @@ def test_thonky_pattern_6():
     # Thonky: 4, but that's impossible: Either 0 or a multiple of 10 (N4 = 10)
     assert 0 == scores[3]
     # See score 3 and 4
-    assert 357 - 4 - 80 + 840 == encoder.evaluate_mask(matrix, matrix_size)
+    assert 357 - 4 - 80 + 840 == encoder.evaluate_mask(matrix, width, height)
 
 
 def test_thonky_pattern_7():
     # http://www.thonky.com/qr-code-tutorial/data-masking
-    matrix = read_matrix('thonky_datamasking_mask-7')
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    matrix, width, height = read_matrix('thonky_datamasking_mask-7')
+    scores = encoder.mask_scores(matrix, width, height)
     assert 197 == scores[0]
     assert 123 == scores[1]
     # Thonky: 200
     assert 720 == scores[2]
     assert 0 == scores[3]
     # See score 3
-    assert 520 - 200 + 720 == encoder.evaluate_mask(matrix, matrix_size)
+    assert 520 - 200 + 720 == encoder.evaluate_mask(matrix, width, height)
 
 
 def test_score_n1_iso():
     # For example, impose 5 penalty points on the block of “dark:dark:dark:dark:dark:dark:dark”
     # module pattern, where a series of seven consecutive modules is counted as one block
     matrix = tuple(bytearray([1] * 7) for i in range(7))
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    width, height = [len(matrix)] * 2
+    scores = encoder.mask_scores(matrix, width, height)
     score = scores[0]
     assert 5 * 7 * 2 == score
 
@@ -758,14 +752,13 @@ def test_score_n2_iso():
     # be included in this block, the penalty applied to this block shall be
     # calculated as 4 (blocks) x 3 (points) = 12 points.
     matrix = (bytearray([1, 1, 1]), bytearray([1, 1, 1]), bytearray([1, 1, 1]))
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    width, height = [len(matrix)] * 2
+    scores = encoder.mask_scores(matrix, width, height)
     score = scores[1]
     assert 12 == score
 
     matrix = (bytearray([0, 0, 0]), bytearray([0, 0, 0]), bytearray([0, 0, 0]))
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    scores = encoder.mask_scores(matrix, width, height)
     score = scores[1]
     assert 12 == score
 
@@ -810,8 +803,8 @@ def test_score_n4_iso(score, percent):
 
     matrix = make_matrix()
     fill_matrix(matrix, percent)
-    matrix_size = len(matrix)
-    scores = encoder.mask_scores(matrix, matrix_size)
+    width, height = [len(matrix)] * 2
+    scores = encoder.mask_scores(matrix, width, height)
     score = scores[3]
     assert score == score
 
@@ -1050,12 +1043,11 @@ def test_make_final_message_thonky():
 def test_encode_iso_fig1():
     # ISO/IEC 18004:2015(E) - page 7
     # 'QR Code Symbol' as 1-M symbol
-    qr = encoder.encode('QR Code Symbol', error='M', mask=None, micro=False,
-                        boost_error=False)
+    qr = encoder.encode('QR Code Symbol', error='M', mask=None, micro=False, boost_error=False)
     assert consts.ERROR_LEVEL_M == qr.error
     assert 1 == qr.version
     assert 5 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
-    ref_matrix = read_matrix('iso-fig-1')
+    ref_matrix = read_matrix('iso-fig-1')[0]
     assert ref_matrix == qr.matrix
 
 
@@ -1064,31 +1056,28 @@ def test_encode_iso_i2():
     # 01234567 as 1-M symbol
     # TODO: Without the mask param Segno chooses mask 3 which seems to be correct
     # Mask 2 is IMO an error in the standard
-    qr = encoder.encode('01234567', error='m', version=1, mask=2, micro=False,
-                        boost_error=False)
+    qr = encoder.encode('01234567', error='m', version=1, mask=2, micro=False, boost_error=False)
     assert consts.ERROR_LEVEL_M == qr.error
     assert 1 == qr.version
     assert 2 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
-    qr = encoder.encode('01234567', error='m', mask=2, micro=False,
-                        boost_error=False)
+    qr = encoder.encode('01234567', error='m', mask=2, micro=False, boost_error=False)
     assert consts.ERROR_LEVEL_M == qr.error
     assert 1 == qr.version
     assert 2 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
-    ref_matrix = read_matrix('iso-i2')
+    ref_matrix = read_matrix('iso-i2')[0]
     assert ref_matrix == qr.matrix
 
 
 def test_encode_iso_i3():
     # ISO/IEC 18004:2015(E) - page 96
     # 01234567 as M2-L symbol
-    ref_matrix = read_matrix('iso-i3')
+    ref_matrix = read_matrix('iso-i3')[0]
     qr = encoder.encode('01234567', error='l', version='m2', boost_error=False)
     assert consts.ERROR_LEVEL_L == qr.error
     assert consts.VERSION_M2 == qr.version
     assert 1 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
     assert ref_matrix == qr.matrix
-    qr = encoder.encode('01234567', error='l', version=None, micro=True,
-                        boost_error=False)
+    qr = encoder.encode('01234567', error='l', version=None, micro=True, boost_error=False)
     assert consts.ERROR_LEVEL_L == qr.error
     assert consts.VERSION_M2 == qr.version
     assert 1 == qr.mask, 'Wrong mask, got: {0}'.format(qr.mask)
@@ -1104,7 +1093,7 @@ def test_encode_iso_fig29():
                         error='m', mask=4, boost_error=False)
     assert 4 == qr.mask
     assert 4 == qr.version
-    ref_matrix = read_matrix('iso-fig-29')
+    ref_matrix = read_matrix('iso-fig-29')[0]
     assert ref_matrix == qr.matrix
 
 
@@ -1115,6 +1104,7 @@ def test_codeword_placement_iso_i2():
         '11101100 00010001 11101100 00010001 11101100 00010001 11101100 00010001'
     codewords = Buffer(bits(s))
     version = 1
+    width, height = 21, 21
     buff = encoder.make_final_message(version, consts.ERROR_LEVEL_M, codewords)
     expected_s = '00010000 00100000 00001100 01010110 01100001 10000000 11101100 ' \
                  '00010001 11101100 00010001 11101100 00010001 11101100 00010001 ' \
@@ -1122,10 +1112,10 @@ def test_codeword_placement_iso_i2():
                  '00110110 11000111 10000111 00101100 01010101'
     expected = bits(expected_s)
     assert expected == buff.getbits()
-    matrix = encoder.make_matrix(version)
-    encoder.add_finder_patterns(matrix, is_micro=version < 1)
+    matrix = encoder.make_matrix(width, height)
+    encoder.add_finder_patterns(matrix, width, height)
     encoder.add_codewords(matrix, buff, version=version)
-    ref_matrix = read_matrix('iso-i2_code_placement')
+    ref_matrix = read_matrix('iso-i2_code_placement')[0]
     assert ref_matrix == matrix
 
 
@@ -1135,118 +1125,111 @@ def test_codeword_placement_iso_i3():
     s = '01000000 00011000 10101100 11000011 00000000'
     codewords = Buffer(bits(s))
     version = consts.VERSION_M2
+    width, height = 13, 13
     buff = encoder.make_final_message(version, consts.ERROR_LEVEL_L, codewords)
     expected_s = '01000000 00011000 10101100 11000011 00000000 10000110 00001101 ' \
                  '00100010 10101110 00110000'
     expected = bits(expected_s)
     assert expected == buff.getbits()
-    matrix = encoder.make_matrix(version)
-    encoder.add_finder_patterns(matrix, is_micro=version < 1)
+    matrix = encoder.make_matrix(width, height)
+    encoder.add_finder_patterns(matrix, width, height)
     encoder.add_codewords(matrix, buff, version=version)
-    ref_matrix = read_matrix('iso-i3_code_placement')
+    ref_matrix = read_matrix('iso-i3_code_placement')[0]
     assert ref_matrix == matrix
 
 
 def _make_figure22_matrix():
-    version = consts.VERSION_M4
-    matrix = encoder.make_matrix(version)
+    width, height = 17, 17  # M4
+    matrix = encoder.make_matrix(width, height)
     for row in matrix:
         for i in range(len(row)):
             if row[i] == 0x2:
                 row[i] = 0x0
-    encoder.add_finder_patterns(matrix, True)
-    return matrix
+    encoder.add_finder_patterns(matrix, width, height)
+    return matrix, width, height
 
 
 def test_figure22_mask0():
     # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
     # Figure 22 - Mask 0
-    version = consts.VERSION_M4
-    matrix = _make_figure22_matrix()
-    mask, matrix = encoder.find_and_apply_best_mask(matrix, version, True,
-                                                    proposed_mask=0)
+    matrix, width, height = _make_figure22_matrix()
+    mask, matrix = encoder.find_and_apply_best_mask(matrix, width, height, proposed_mask=0)
     assert 0 == mask
     # Format info = dark modules
     for i in range(9):
         matrix[8][i] = 0x1
         matrix[i][8] = 0x1
-    ref_matrix = read_matrix('fig-22-mask-0')
-    assert len(ref_matrix) == len(matrix)
+    ref_matrix, ref_width, ref_height = read_matrix('fig-22-mask-0')
+    assert ref_width, ref_height == (width, height)
     assert ref_matrix == matrix
 
 
 def test_figure22_mask1():
     # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
     # Figure 22 - Mask 1
-    version = consts.VERSION_M4
-    matrix = _make_figure22_matrix()
-    mask, matrix = encoder.find_and_apply_best_mask(matrix, version, True,
-                                                    proposed_mask=1)
+    matrix, width, height = _make_figure22_matrix()
+    mask, matrix = encoder.find_and_apply_best_mask(matrix, width, height, proposed_mask=1)
     assert 1 == mask
     # Format info = dark modules
     for i in range(9):
         matrix[8][i] = 0x1
         matrix[i][8] = 0x1
-    ref_matrix = read_matrix('fig-22-mask-1')
-    assert len(ref_matrix) == len(matrix)
+    ref_matrix, ref_width, ref_height = read_matrix('fig-22-mask-1')
+    assert ref_width, ref_height == (width, height)
     assert ref_matrix == matrix
 
 
 def test_figure22_mask2():
     # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
     # Figure 22 - Mask 2
-    version = consts.VERSION_M4
-    matrix = _make_figure22_matrix()
-    mask, matrix = encoder.find_and_apply_best_mask(matrix, version, True,
-                                                    proposed_mask=2)
+    matrix, width, height = _make_figure22_matrix()
+    mask, matrix = encoder.find_and_apply_best_mask(matrix, width, height, proposed_mask=2)
     assert 2 == mask
     # Format info = dark modules
     for i in range(9):
         matrix[8][i] = 0x1
         matrix[i][8] = 0x1
-    ref_matrix = read_matrix('fig-22-mask-2')
-    assert len(ref_matrix) == len(matrix)
+    ref_matrix, ref_width, ref_height = read_matrix('fig-22-mask-2')
+    assert ref_width, ref_height == (width, height)
     assert ref_matrix == matrix
 
 
 def test_figure22_mask3():
     # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
     # Figure 22 - Mask 3
-    version = consts.VERSION_M4
-    matrix = _make_figure22_matrix()
-    mask, matrix = encoder.find_and_apply_best_mask(matrix, version, True,
-                                                    proposed_mask=3)
+    matrix, width, height = _make_figure22_matrix()
+    mask, matrix = encoder.find_and_apply_best_mask(matrix, width, height, proposed_mask=3)
     assert 3 == mask
     # Format info = dark modules
     for i in range(9):
         matrix[8][i] = 0x1
         matrix[i][8] = 0x1
-    ref_matrix = read_matrix('fig-22-mask-3')
-    assert len(ref_matrix) == len(matrix)
+    ref_matrix, ref_width, ref_height = read_matrix('fig-22-mask-3')
+    assert ref_width, ref_height == (width, height)
     assert ref_matrix == matrix
 
 
 def test_fig23_best_mask():
     # ISO/IEC 18004:2015(E) - 7.8.2 Data mask patterns
     # Figure 23
-    matrix = read_matrix('fig-23-unmasked')
-    mask, matrix = encoder.find_and_apply_best_mask(matrix, 1, is_micro=False)
+    matrix, width, height = read_matrix('fig-23-unmasked')
+    mask, matrix = encoder.find_and_apply_best_mask(matrix, width, height)
     assert 0 == mask
-    ref_matrix = read_matrix('fig-23-mask-0')
+    ref_matrix, ref_width, ref_height = read_matrix('fig-23-mask-0')
+    assert ref_width, ref_height == (width, height)
     assert ref_matrix == matrix
 
 
 def test_format_info_figure26():
     # 7.9.2 Micro QR Code symbols (page 57)
     version = consts.VERSION_M1
+    width, height = 10, 10
     mask = 3
     matrix = tuple([bytearray([0x0] * 11) for i in range(11)])
     encoder.add_timing_pattern(matrix, is_micro=True)
-    encoder.add_finder_patterns(matrix, is_micro=True)
-    encoder.add_format_info(matrix, version=version, error=None,
-                            mask_pattern=mask)
-    ref_matrix = read_matrix('fig-26')
-    assert len(ref_matrix) == len(matrix)
+    encoder.add_finder_patterns(matrix, width, height)
+    encoder.add_format_info(matrix, version=version, error=None, mask_pattern=mask)
+    ref_matrix = read_matrix('fig-26')[0]
     assert ref_matrix == matrix
 
 
