@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2016 - 2022 -- Lars Heuer
+# Copyright (c) 2016 - 2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -15,7 +15,8 @@ from segno import consts, writers
 
 
 def test_issue71():
-    cm = writers._make_colormap(7, dark='blue', light='white', finder_dark=None, finder_light=None)
+    width, height = 45, 45  # Version 7
+    cm = writers._make_colormap(width, height, dark='blue', light='white', finder_dark=None, finder_light=None)
     assert cm[consts.TYPE_FINDER_PATTERN_DARK] is None
     assert cm[consts.TYPE_FINDER_PATTERN_LIGHT] is None
 
