@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2022 -- Lars Heuer
+# Copyright (c) 2022 -2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -11,14 +11,9 @@ Test against issue #109.
 
 Requires pyzbar and additional libs (libzbar0).
 """
-from __future__ import absolute_import, unicode_literals
 import io
 import pytest
 import segno
-try:
-    FileNotFoundError
-except NameError:  # Py2
-    FileNotFoundError = OSError
 try:
     from pyzbar.pyzbar import decode as zbardecode
 except (ImportError, FileNotFoundError):  # The latter may occur under Windows
