@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2016 - 2022 -- Lars Heuer
+# Copyright (c) 2016 - 2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -8,15 +8,10 @@
 """\
 PAM related tests.
 """
-from __future__ import unicode_literals, absolute_import
 import re
 import io
 import pytest
 import segno
-try:
-    range = xrange  # noqa
-except NameError:
-    pass
 
 
 def test_invalid_color():
