@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2016 - 2022 -- Lars Heuer
+# Copyright (c) 2016 - 2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -14,10 +14,6 @@ import io
 from struct import unpack
 import pytest
 import segno
-try:
-    range = xrange  # noqa
-except NameError:
-    pass
 
 
 def test_invalid_color():
