@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2016 - 2022 -- Lars Heuer
+# Copyright (c) 2016 - 2023 -- Lars Heuer
 # All rights reserved.
 #
 # License: BSD License
@@ -10,50 +10,51 @@ Tests against the ``utils`` module.
 """
 from __future__ import absolute_import, unicode_literals
 import pytest
-from segno import utils, consts
+from segno import utils
 
 
-def test_get_border():
-    border = utils.get_border(1, None)
+def test_get_border_qr():
+    matrix_size = 21, 21  # Version 1
+    border = utils.get_border(matrix_size, None)
     assert 4 == border
-    border = utils.get_border(1, 3)
+    border = utils.get_border(matrix_size, 3)
     assert 3 == border
 
 
 def test_get_border2():
-    border = utils.get_border(consts.VERSION_M1, 1)
+    matrix_size = 11, 11  # M1
+    border = utils.get_border(matrix_size, 1)
     assert 1 == border
-    border = utils.get_border(consts.VERSION_M1, None)
+    border = utils.get_border(matrix_size, None)
     assert 2 == border
 
 
 def test_get_border3():
-    border = utils.get_border(3, 0)
+    matrix_size = 29, 29  # Version 3
+    border = utils.get_border(matrix_size, 0)
     assert 0 == border
-    border = utils.get_border(3, None)
+    border = utils.get_border(matrix_size, None)
     assert 4 == border
 
 
 def test_get_symbol_size():
-    version = 1
-    matrix_size = 21
+    matrix_size = 21, 21  # QR Code version 1
     border = 0
-    width, height = utils.get_symbol_size(version, border=border)
-    assert (matrix_size, matrix_size) == (width, height)
+    width, height = utils.get_symbol_size(matrix_size, border=border)
+    assert matrix_size == (width, height)
     border = 4
-    width, height = utils.get_symbol_size(1)
-    assert (matrix_size + 2 * border, matrix_size + 2 * border) == (width, height)
+    width, height = utils.get_symbol_size(matrix_size)
+    assert (matrix_size[0] + 2 * border, matrix_size[1] + 2 * border) == (width, height)
 
 
 def test_get_symbol_size_micro():
-    version = consts.VERSION_M2
-    matrix_size = 13
+    matrix_size = 13, 13  # M2
     border = 0
-    width, height = utils.get_symbol_size(version, border=border)
-    assert (matrix_size, matrix_size) == (width, height)
+    width, height = utils.get_symbol_size(matrix_size, border=border)
+    assert matrix_size == (width, height)
     border = 2
-    width, height = utils.get_symbol_size(version)
-    assert (matrix_size + 2 * border, matrix_size + 2 * border) == (width, height)
+    width, height = utils.get_symbol_size(matrix_size)
+    assert (matrix_size[0] + 2 * border, matrix_size[1] + 2 * border) == (width, height)
 
 
 @pytest.mark.parametrize('scale', [1, 1.2, .8, 10])
